@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,18 +25,30 @@
 #include <sys/types.h>
 #include "config.h"
 
-#define	_PED_SWAP16(x)		( (((x) & 0x00ff) << 8)		\
-				+ (((x) & 0xff00) >> 8) )
 
-#define	_PED_SWAP32(x)		( (((x) & 0x000000ff) << 24)	\
-				+ (((x) & 0x0000ff00) << 8)	\
-				+ (((x) & 0x00ff0000) >> 8)	\
-				+ (((x) & 0xff000000) >> 24) )
+/* returns the n'th least significant byte */
+#define _GET_BYTE(x, n)		( ((x) >> (8 * (n))) & 0xff )
 
-#define PED_SWAP16(x)		((unsigned short)	\
-					_PED_SWAP16( (unsigned short) x ))
-#define PED_SWAP32(x)		((unsigned int)		\
-					_PED_SWAP32( (unsigned int) x ))
+#define _PED_SWAP16(x)		( (_GET_BYTE(x, 0) << 8)	\
+				+ (_GET_BYTE(x, 1) << 0) )
+
+#define _PED_SWAP32(x)		( (_GET_BYTE(x, 0) << 24)	\
+				+ (_GET_BYTE(x, 1) << 16)	\
+				+ (_GET_BYTE(x, 2) << 8)	\
+				+ (_GET_BYTE(x, 3) << 0) )
+
+#define _PED_SWAP64(x)		( (_GET_BYTE(x, 0) << 56)	\
+				+ (_GET_BYTE(x, 1) << 48)	\
+				+ (_GET_BYTE(x, 2) << 40)	\
+				+ (_GET_BYTE(x, 3) << 32)	\
+				+ (_GET_BYTE(x, 4) << 24)	\
+				+ (_GET_BYTE(x, 5) << 16)	\
+				+ (_GET_BYTE(x, 6) << 8)	\
+				+ (_GET_BYTE(x, 7) << 0) )
+
+#define PED_SWAP16(x)		((__u16) _PED_SWAP16( (__u16) x ))
+#define PED_SWAP32(x)		((__u32) _PED_SWAP32( (__u32) x ))
+#define PED_SWAP64(x)		((__u64) _PED_SWAP64( (__u64) x ))
 
 #ifdef WORDS_BIGENDIAN
 
@@ -44,11 +56,15 @@
 #define PED_CPU_TO_BE16(x)	(x)
 #define PED_CPU_TO_LE32(x)	PED_SWAP32(x)
 #define PED_CPU_TO_BE32(x)	(x)
+#define PED_CPU_TO_LE64(x)	PED_SWAP64(x)
+#define PED_CPU_TO_BE64(x)	(x)
 
 #define PED_LE16_TO_CPU(x)	PED_SWAP16(x)
 #define PED_BE16_TO_CPU(x)	(x)
 #define PED_LE32_TO_CPU(x)	PED_SWAP32(x)
 #define PED_BE32_TO_CPU(x)	(x)
+#define PED_LE64_TO_CPU(x)	PED_SWAP32(x)
+#define PED_BE64_TO_CPU(x)	(x)
 
 #else /* !WORDS_BIGENDIAN */
 
@@ -56,11 +72,15 @@
 #define PED_CPU_TO_BE16(x)	PED_SWAP16(x)
 #define PED_CPU_TO_LE32(x)	(x)
 #define PED_CPU_TO_BE32(x)	PED_SWAP32(x)
+#define PED_CPU_TO_LE64(x)	(x)
+#define PED_CPU_TO_BE64(x)	PED_SWAP64(x)
 
 #define PED_LE16_TO_CPU(x)	(x)
 #define PED_BE16_TO_CPU(x)	PED_SWAP16(x)
 #define PED_LE32_TO_CPU(x)	(x)
 #define PED_BE32_TO_CPU(x)	PED_SWAP32(x)
+#define PED_LE64_TO_CPU(x)	(x)
+#define PED_BE64_TO_CPU(x)	PED_SWAP64(x)
 
 #endif /* !WORDS_BIGENDIAN */
 
