@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1998-2000 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -62,14 +62,14 @@ extern void ped_file_system_type_unregister (PedFileSystemType* type);
 
 extern PedFileSystemType* ped_file_system_type_get (const char* name);
 extern PedFileSystemType*
-ped_file_system_type_get_next (PedFileSystemType* fs_type);
+ped_file_system_type_get_next (const PedFileSystemType* fs_type);
 
 extern PedFileSystemType* ped_file_system_probe (const PedGeometry* geom);
 extern int ped_file_system_clobber (PedGeometry* geom);
 
 extern PedFileSystem* ped_file_system_open (PedGeometry* geom);
 extern PedFileSystem* ped_file_system_create (PedGeometry* geom,
-					      PedFileSystemType* type);
+					      const PedFileSystemType* type);
 extern int ped_file_system_close (PedFileSystem* fs);
 extern int ped_file_system_check (PedFileSystem* fs);
 extern int ped_file_system_copy (const PedFileSystem* fs, PedGeometry* geom);
