@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -725,20 +725,20 @@ read_table (PedDisk* disk, PedSector sector, int is_extended_table)
 		if (raw_part->type == 0x0 || !raw_part->length)
 			continue;
 
-		if (is_extended_table) {
-			if (raw_part_is_extended (raw_part))
-				lba_offset = ped_disk_extended_partition
-						(disk)->geom.start;
-			else
-				lba_offset = sector;
-		} else {
+		/* process nested extended partitions after normal logical
+		 * partitions, to make sure we get the order right.
+		 */
+		if (is_extended_table && raw_part_is_extended (raw_part))
+			continue;	
+
+		if (is_extended_table)
+			lba_offset = sector;
+		else
 			lba_offset = 0;
-		}
 
 		if (!check_consistancy (disk, raw_part, lba_offset))
 			return 0;
 		part_start = linear_start (disk, raw_part, lba_offset);
-
 		if (part_start == sector) {
 			if (ped_exception_throw (
 				PED_EXCEPTION_ERROR,
@@ -751,22 +751,12 @@ read_table (PedDisk* disk, PedSector sector, int is_extended_table)
 			continue;	/* avoid infinite recursion */
 		}
 
-		if (is_extended_table) {
-			if (raw_part_is_extended (raw_part)) {
-				/* hide M$ brain-damage ala fdisk, by not
-				   adding nested extended partitions in a tree.
-				   NOTE: sector offset is in terms of the
-				   first extended partition */
-				if (!read_table (disk, part_start, 1))
-					return 0;
-				continue;
-			}
+		if (is_extended_table)
 			type = PED_PARTITION_LOGICAL;
-		} else {
-			type = (raw_part_is_extended (raw_part))
-				? PED_PARTITION_EXTENDED
-				: PED_PARTITION_PRIMARY;
-		}
+		else if (raw_part_is_extended (raw_part))
+			type = PED_PARTITION_EXTENDED;
+		else
+			type = PED_PARTITION_PRIMARY;
 
 		part = ped_partition_new (
 			disk, type, NULL,
@@ -797,6 +787,21 @@ read_table (PedDisk* disk, PedSector sector, int is_extended_table)
 		}
 	}
 
+	if (is_extended_table) {
+		/* process the nested extended partitions */
+		for (i = 0; i < 4; i++) {
+			raw_part = &table.partitions [i];
+			if (!raw_part_is_extended (raw_part))
+				continue;
+
+			lba_offset = ped_disk_extended_partition
+					(disk)->geom.start;
+			part_start = linear_start (disk, raw_part, lba_offset);
+			if (!read_table (disk, part_start, 1))
+				return 0;
+		}
+	}
+
 	return 1;
 
 error:
