@@ -19,6 +19,8 @@
 
 #include "config.h"
 
+#ifdef ENABLE_PC98
+
 #include <parted/parted.h>
 #include <parted/endian.h>
 #include <parted/disk_pc98.h>
@@ -929,3 +931,5 @@ pc98_get_max_primary_partition_count (const PedDisk* disk)
 {
 	return MAX_PART_COUNT;
 }
+
+#endif /* ENABLE_PC98 */
