@@ -221,7 +221,7 @@ ped_file_system_open (PedGeometry* geom)
 }
 
 PedFileSystem*
-ped_file_system_create (PedGeometry* geom, PedFileSystemType* type)
+ped_file_system_create (PedGeometry* geom, const PedFileSystemType* type)
 {
 	PED_ASSERT (geom != NULL, return NULL);
 	PED_ASSERT (type != NULL, return NULL);
