@@ -259,9 +259,6 @@ static int dotindblock(struct ext2_fs *fs,
 
 
 
-/* Ehrm.... sorry, pedanticists! :-) */
-#define offsetof(type, field) ((size_t)(&(((type *)0)->field)))
-
 static int doinode(struct ext2_fs *fs, struct ext2_block_relocator_state *state, int inode)
 {
 	struct ext2_inode buf;
@@ -277,6 +274,20 @@ static int doinode(struct ext2_fs *fs, struct ext2_block_relocator_state *state,
 		blk_t inodeblock;
 
 		inodeoffset = ext2_get_inode_offset(fs, inode, &inodeblock);
+
+		/* do Hurd block, if there is one... */
+		if (EXT2_SUPER_CREATOR_OS(fs->sb) == EXT2_OS_HURD
+		    && EXT2_INODE_TRANSLATOR(buf)) {
+			if (!doblock(fs,
+				     state,
+				     EXT2_INODE_TRANSLATOR(buf),
+				     inodeblock,
+				     inodeoffset + offsetof(struct ext2_inode,
+						osd1.hurd1.h_i_translator),
+				     0))
+				return 0;
+		}
+
 		for (i=0;i<EXT2_NDIR_BLOCKS;i++)
 			if ((blk = EXT2_INODE_BLOCK(buf, i)) != 0)
 				if (!doblock(fs,
