@@ -1,6 +1,6 @@
 /*
     ext2_mkfs.c -- ext2 fs creator
-    Copyright (C) 1998-2000 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
   
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -78,9 +78,11 @@ static int ext2_mkfs_write_main(struct ext2_dev_handle *handle,
 	int numgroups;
 	int gdblocks;
 	unsigned char *sbbuf;
+	struct ext2_super_block *sb_for_io;
 
 	freeit = 0;
 	sbbuf = (unsigned char *)sb;
+	sb_for_io = sb;
 	if (EXT2_SUPER_LOG_BLOCK_SIZE(*sb))
 	{
 		sbbuf = ped_malloc(1024 << EXT2_SUPER_LOG_BLOCK_SIZE(*sb));
@@ -88,6 +90,7 @@ static int ext2_mkfs_write_main(struct ext2_dev_handle *handle,
 			return 0;
 		memcpy(sbbuf+1024, sb, 1024);
 		freeit = 1;
+		sb_for_io = (struct ext2_super_block*) (sbbuf + 1024);
 	}
 
 	numgroups = howmany(EXT2_SUPER_BLOCKS_COUNT(*sb)
@@ -105,6 +108,8 @@ static int ext2_mkfs_write_main(struct ext2_dev_handle *handle,
 			offset = EXT2_SUPER_FIRST_DATA_BLOCK(*sb)
 				 + i * EXT2_SUPER_BLOCKS_PER_GROUP(*sb);
 
+			sb_for_io->s_block_group_nr = PED_CPU_TO_LE16 (i);
+
 			if (!handle->ops->write(handle->cookie, sbbuf,
 					        offset, 1))
 				return 0;
@@ -114,6 +119,8 @@ static int ext2_mkfs_write_main(struct ext2_dev_handle *handle,
 		}
 	}
 
+	sb_for_io->s_block_group_nr = 0;
+
 	if (freeit)
 		ped_free(sbbuf);
 	return 1;
@@ -346,7 +353,6 @@ static int ext2_mkfs_create_root_inode(struct ext2_fs *fs)
 {
 	struct ext2_buffer_head *bh;
 	blk_t block;
-	__u32 data[fs->blocksize / 4];
 	struct ext2_inode inode;
 	int offset;
 
@@ -356,15 +362,14 @@ static int ext2_mkfs_create_root_inode(struct ext2_fs *fs)
 		return 0;
 
 	/* create directory entries */
-	memset(data, 0, fs->blocksize);
-	offset = _set_dirent(data, 0, fs->blocksize, 0,
-			     EXT2_ROOT_INO, ".", EXT2_FT_DIR);
-	offset = _set_dirent(data, offset, fs->blocksize, 0,
-			     EXT2_ROOT_INO, "..", EXT2_FT_DIR);
-	offset = _set_dirent(data, offset, fs->blocksize, 1,
-			     11, "lost+found", EXT2_FT_DIR);
 	bh = ext2_bcreate(fs, block);
-	memcpy(bh->data, data, fs->blocksize);
+	memset(bh->data, 0, fs->blocksize);
+	offset = _set_dirent(bh->data, 0, fs->blocksize, 0,
+			     EXT2_ROOT_INO, ".", EXT2_FT_DIR);
+	offset = _set_dirent(bh->data, offset, fs->blocksize, 0,
+			     EXT2_ROOT_INO, "..", EXT2_FT_DIR);
+	offset = _set_dirent(bh->data, offset, fs->blocksize, 1,
+			     11, "lost+found", EXT2_FT_DIR);
 	bh->dirty = 1;
 	if (!ext2_brelse(bh, 1))
 		return 0;
@@ -422,7 +427,7 @@ static int ext2_mkfs_init_sb (struct ext2_super_block *sb, blk_t numblocks,
 
 	/* hack: this get's inc'd as we go through each group in
 	 * ext2_mkfs_write_meta()
-	*/
+	 */
 	sb->s_free_blocks_count = 0;
 	sb->s_free_inodes_count = PED_CPU_TO_LE32 (numgroups
 							* inodes_per_group);
@@ -488,15 +493,21 @@ struct ext2_fs *ext2_mkfs(struct ext2_dev_handle *handle,
 	int last_group_blocks;
 	int last_group_admin;
 
-	if (log_block_size == 0)
-		log_block_size = 12;
+	/* if the FS is > 512Mb, use 4k blocks, otherwise 1k blocks */
+	if (log_block_size == 0) {
+		handle->ops->set_blocksize(handle->cookie, 12);
+		if (handle->ops->get_size(handle->cookie) > (512 * 1024))
+			log_block_size = 12;
+		else
+			log_block_size = 10;
+	}
 
 	handle->ops->set_blocksize(handle->cookie, log_block_size);
 
 	if (numblocks == 0)
 		numblocks = handle->ops->get_size(handle->cookie);
 
-	if (blocks_per_group == 0)
+	if (blocks_per_group == (unsigned int) 0)
 		blocks_per_group = 8 << log_block_size;
 
 	first_block = (log_block_size == 10) ? 1 : 0;
