@@ -24,6 +24,7 @@
 #include <parted/disk_loop.h>
 #include <parted/disk_pc98.h>
 #include <parted/disk_mac.h>
+#include <parted/disk_gpt.h>
 
 #include "fat.h"
 #include "calc.h"
@@ -586,6 +587,23 @@ fat_check (PedFileSystem* fs)
 	return 1;	/* existence of fs implies consistency ;-) */
 }
 
+/* Calculates how much space there will be in clusters in:
+ * 	old_fs intersect the-new-fs
+ */
+static PedSector
+_calc_resize_data_size (
+	const PedFileSystem* old_fs,
+	PedSector new_cluster_sectors,
+	FatCluster new_cluster_count,
+	PedSector new_fat_size)
+{
+	FatSpecific*	old_fs_info = FAT_SPECIFIC (old_fs);
+	PedSector	fat_size_delta;
+
+	fat_size_delta = old_fs_info->fat_sectors - new_fat_size;
+	return new_cluster_sectors * new_cluster_count - fat_size_delta * 2;
+}
+
 static int
 _test_size (const PedFileSystem* fs, PedSector length, PedSector min_data_size)
 {
@@ -606,7 +624,9 @@ _test_size (const PedFileSystem* fs, PedSector length, PedSector min_data_size)
 				&_cluster_sectors,
 				&_cluster_count,
 				&_fat_size)
-			&& _cluster_sectors * _cluster_count >= min_data_size)
+	    && _calc_resize_data_size (fs, _cluster_sectors, _cluster_count,
+		    		       _fat_size)
+	    		>= min_data_size)
 		return 1;
 
 	if (fat_calc_resize_sizes (
@@ -618,7 +638,9 @@ _test_size (const PedFileSystem* fs, PedSector length, PedSector min_data_size)
 				&_cluster_sectors,
 				&_cluster_count,
 				&_fat_size)
-			&& _cluster_sectors * _cluster_count >= min_data_size)
+	    && _calc_resize_data_size (fs, _cluster_sectors, _cluster_count,
+		    		       _fat_size)
+	    		>= min_data_size)
 		return 1;
 
 	return 0;
@@ -642,7 +664,10 @@ _get_min_size (const PedFileSystem* fs, PedSector min_data_size)
 			min_length = length;
 	}
 
-	return max_length;
+/* adds a bit of leeway (64 sectors), for resolving extra issues, like root
+ * directory allocation, that aren't covered here. 
+ */
+	return max_length + 64;
 }
 
 PedConstraint*
@@ -663,13 +688,10 @@ fat_get_resize_constraint (const PedFileSystem* fs)
 	min_cluster_count = used_clusters + fs_info->total_dir_clusters;
 	min_data_size = min_cluster_count * fs_info->cluster_sectors;
 
-/* adds a bit of leeway (32 sectors), for resolving extra issues, like root
- * directory allocation, that aren't covered here. 
- */
 	return ped_constraint_new (ped_alignment_any, ped_alignment_any,
 				   &full_disk,
 				   &full_disk,
-				   _get_min_size (fs, min_data_size) + 32);
+				   _get_min_size (fs, min_data_size));
 }
 
 int
@@ -770,6 +792,15 @@ fat_set_system (const PedFileSystem* fs, PedPartition* part,
 		return 1;
 	}
 
+	if (strcmp (disk_type->name, GPT_NAME) == 0) {
+		GPTPartitionData *gpt_part_data = part->disk_specific;
+		PED_ASSERT(gpt_part_data      != NULL, return 0);	
+		PED_ASSERT(gpt_part_data->pte != NULL, return 0);
+		gpt_part_data->pte->PartitionTypeGuid =
+			PARTITION_BASIC_DATA_GUID;
+		return 1;
+	}
+
 	return 0;
 }
 
