@@ -638,6 +638,9 @@ create_resize_context (PedFileSystem* fs, const PedGeometry* new_geom)
 	new_fs_info->fat_table_count = 2;
 	new_fs_info->fat_sectors = new_fat_sectors;
 
+	/* what about copying? */
+	new_fs_info->serial_number = fs_info->serial_number;
+
 	if (new_fs_info->fat_type == FAT_TYPE_FAT32) {
 		new_fs_info->info_sector_offset	= 1;
 		new_fs_info->boot_sector_backup_offset = 6;
