@@ -25,6 +25,7 @@
 #include <parted/disk_loop.h>
 #include <parted/disk_pc98.h>
 #include <parted/disk_mac.h>
+#include <parted/disk_gpt.h>
 
 #include <libintl.h>
 #if ENABLE_NLS
@@ -33,8 +34,6 @@
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
 
-#include <unistd.h>
-#include <asm/page.h>
 #include <string.h>
 
 #define HFS_SIGNATURE	0x4244
@@ -154,6 +153,15 @@ hfs_set_system (const PedFileSystem* fs, PedPartition* part,
 		return 1;
 	}
 
+	if (strcmp (disk_type->name, GPT_NAME) == 0) {
+		GPTPartitionData *gpt_part_data = part->disk_specific;
+		PED_ASSERT(gpt_part_data      != NULL, return 0);	
+		PED_ASSERT(gpt_part_data->pte != NULL, return 0);
+		gpt_part_data->pte->PartitionTypeGuid =
+			PARTITION_BASIC_DATA_GUID;
+		return 1;
+	}
+
 	return 0;
 }
 
